/*
 * Demonstrating Buffer Overflows
 *
 * Examples presented at the Grazer Linuxtage 2004
 *
 * Copyright (C) 2004 Thomas Winkler <tom@qwws.net>
 * Copyright (C) 2004 Markus Quaritsch <quam@qwws.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */


/* 
 * When you look at the main-function one would expect that the output of this
 * program will be '1' (x=1 just before the printf statement)
 *
 * But in the function we modify the return address to jump over the assignment.
 * The output of the program is '0'!
 */

void function (int a, int b, int c) {
	/* local variables placed on the stack */
	char buffer1[5];
	char buffer2[10];
	int *ret;

	/* let ret point to the return-address of this function. Therefore we take
	 * the address of buffer1 and increment it by a certain value. This value
	 * depends on the compiler.
	 */
	ret = (int *)buffer1 + 7;	/* here we increment the address of buffer1 by 
								   28 bytes (5 bytes for buffer1 and some space 
								   reserved by the compiler 
								 */
	(*ret) += 7;				/* afterwards we increment the return address by 
								   the number of bytes used for the instruction we
								   want to jump over - here 7 bytes.
								 */
}

int main() {

	int x;

	x = 0;
	function (1, 2, 3);
	x = 1;					/* this statement will not be executed */
	printf("%d\n", x);
}
